﻿# -*- coding=utf-8 -*-

import math, sys
sys.path.append("D:\Programs\ActivePython\site-packages")
import MySQLdb

# classe d'exception
class ImpotsError:
    pass

# --------------------------------------------------------------------------
class Utilitaires:
    """classe de fonctions utilitaires"""
    def cutNewLineChar(self,ligne):
        # on supprime la marque de fin de ligne de ligne si elle existe
        l=len(ligne)
        while(ligne[l-1]=="\n" or ligne[l-1]=="\r"):
            l-=1
        return(ligne[0:l])

# --------------------------------------------------------------------------
class ImpotsFile:
    def __init__(self,IMPOTS):
        # IMPOTS : le nom du fichier contenant les données des tables limites, coeffR, coeffN
        # on ouvre le fichier
        data=open(IMPOTS,"r")
        # un objet Utilitaires
        u=Utilitaires()
        # création des 3 tables - on suppose que les 3 lignes de IMPOTS sont syntaxiquement correctes
        # -- ligne 1
        ligne=data.readline()
        if ligne== '':
            raise ImpotsError("La premiere ligne du fichier {0} est absente".format(IMPOTS))
        limites=u.cutNewLineChar(ligne).split(":")
        for i in range(len(limites)):
            limites[i]=int(limites[i])
        # -- ligne 2
        ligne=data.readline()
        if ligne== '':
            raise ImpotsError("La deuxieme ligne du fichier {0} est absente".format(IMPOTS))
        coeffR=u.cutNewLineChar(ligne).split(":")
        for i in range(len(coeffR)):
            coeffR[i]=float(coeffR[i])
        # -- ligne 3
        ligne=data.readline()
        if ligne== '':
            raise ImpotsError("La troisieme ligne du fichier {0} est absente".format(IMPOTS))
        coeffN=u.cutNewLineChar(ligne).split(":")
        for i in range(len(coeffN)):
            coeffN[i]=float(coeffN[i])
        # fin
        (self.limites,self.coeffR,self.coeffN)=(limites,coeffR,coeffN)

    def getData(self):
        return (self.limites, self.coeffR, self.coeffN)

class ImpotsMetier:

    # constructeur
    # on récupère un pointeur sur la couche [dao]
    def __init__(self, dao):
        self.dao=dao

    # calcul de l'impôt
    # --------------------------------------------------------------------------
    def calculer(self,marie,enfants,salaire):
        # marié : oui, non
        # enfants : nombre d'enfants
        # salaire : salaire annuel

        # on demande à la couche [dao] les données nécessaires au calcul
        (limites, coeffR, coeffN)=self.dao.getData()

       # nombre de parts
        marie=marie.lower()
        if(marie=="oui"):
            nbParts=float(enfants)/2+2
        else:
            nbParts=float(enfants)/2+1
        # une 1/2 part de plus si au moins 3 enfants
        if enfants>=3:
            nbParts+=0.5
        # revenu imposable
        revenuImposable=0.72*salaire
        # quotient familial
        quotient=revenuImposable/nbParts
        # est mis à la fin du tableau limites pour arrêter la boucle qui suit
        limites[len(limites)-1]=quotient
        # calcul de l'impôt
        i=0
        while quotient>limites[i] :
            i=i+1
        # du fait qu'on a placé quotient à la fin du tableau limites, la boucle précédente
        # ne peut déborder du tableau limites
        # maintenant on peut calculer l'impôt
        return math.floor(revenuImposable*(float)(self.dao.coeffR[i])-nbParts*(float)(self.dao.coeffN[i]))

class ImpotsMySQL:

    # constructeur
    def __init__(self,HOTE,USER,PWD,BASE,TABLE):
        # initialise les attributs limites, coeffR, coeffN
        # les données nécessaires au calcul de l'impôt ont été placées dans la table mysqL TABLE
        # appartenant à la base BASE. La table a la structure suivante
        # limites decimal(10,2), coeffR decimal(10,2), coeffN decimal(10,2)
        # la connexion à la base mysql de la machine HOTE se fait sous l'identité (USER,PWD)
        # lance une exception si éventuelle erreur
        
        # connexion à la base mysql
        connexion=MySQLdb.connect(host=HOTE,user=USER,passwd=PWD,db=BASE)
        # on demande un curseur
        curseur=connexion.cursor()
        
        # lecture en bloc de la table TABLE
        requete="select limites,coeffR,coeffN from %s" % (TABLE)
        # exécute la requête [requete] sur la base [base] de la connexion [connexion]
        curseur.execute(requete)
        # exploitation du résultat de la requête
        ligne=curseur.fetchone()
        self.limites=[]
        self.coeffR=[]
        self.coeffN=[]
        while(ligne):
            # ligne courante
            self.limites.append(ligne[0])
            self.coeffR.append(ligne[1])
            self.coeffN.append(ligne[2])
            # ligne suivante
            ligne=curseur.fetchone()
        # déconnexion
        connexion.close()

    def getData(self):
        return (self.limites, self.coeffR, self.coeffN)
