﻿# -*- coding=utf-8 -*-

import sys,socket

#-----------------------------------------------------------------------
def getInfos(fichier):
    # rend les informations (smtp,expéditeur,destinataire,message) prises dans le fichier texte [fichier]
    # ligne 1 : smtp, expéditeur, destinataire
    # lignes suivantes : le texte du message

    # ouverture de [fichier]
    erreur=""
    try:
        infos=open(fichier,"r")
    except IOError, erreur:
        return ("Le fichier %s n'a pu etre ouvert en lecture : %s" % (fichier, erreur))
  
    # lecture de la 1ère ligne
    ligne=infos.readline()
    # suppression de la marque de fin de ligne
    ligne=cutNewLineChar(ligne)
    # récupération des champs smtp,expéditeur,destinataire
    champs=ligne.split(",")
    # a-t-on le bon nombre de champs ?
    if len(champs)!=3 :
        return ("La ligne 1 du fichier %s (serveur smtp, expediteur, destinataire) a un nombre de champs incorrect" % (fichier))
    # "traitement" des informations récupérées
    # on enléve à chacun des 3 champs les "blancs" qui précèdent ou suivent l'information utile
    for i in range(3):
        champs[i]=champs[i].strip()
    # récupération des champs
    (smtpServer,expediteur,destinataire)=champs
    message=""
    # lecture reste du message
    ligne=infos.readline()
    while ligne!='':
        message+=ligne
        ligne=infos.readline()
    infos.close()
    # retour
    return ("",smtpServer,expediteur,destinataire,message)

#-----------------------------------------------------------------------
def sendmail(smtpServer,expediteur,destinataire,message,verbose):
    # envoie message au serveur smtp smtpserver de la part de expéditeur
    # pour destinataire. Si verbose=True, fait un suivi des échanges client-serveur

    # on récupère le nom du client
    try:
        client=socket.gethostbyaddr(socket.gethostbyname("localhost"))[0]
    except socket.error, erreur:
        return "Erreur IP / Nom du client : %s" % (erreur)
    
    # ouverture d'une connexion sur le port 25 de smtpServer
    try:
        connexion=socket.create_connection((smtpServer,25))
    except socket.error, erreur:
        return "Echec de la connexion au site (%s,25) : %s" % (smtpServer,erreur)

    # connexion représente un flux de communication bidirectionnel
    # entre le client (ce programme) et le serveur smtp contacté
    # ce canal est utilisé pour les échanges de commandes et d'informations

    # après la connexion le serveur envoie un message de bienvenue qu'on lit
    erreur=sendCommand(connexion,"",verbose,1)
    if(erreur) :
        connexion.close()
        return erreur
    # cmde ehlo:
    erreur=sendCommand(connexion,"EHLO %s" % (client),verbose,1)
    if erreur :
        connexion.close()
        return erreur
    # cmde mail from:
    erreur=sendCommand(connexion,"MAIL FROM: <%s>" % (expediteur),verbose,1)
    if erreur :
        connexion.close()
        return erreur
    # cmde rcpt to:
    erreur=sendCommand(connexion,"RCPT TO: <%s>" % (destinataire),verbose,1)
    if erreur :
        connexion.close()
        return erreur
    # cmde data
    erreur=sendCommand(connexion,"DATA",verbose,1)
    if erreur :
        connexion.close()
        return erreur
    # préparation message à envoyer
    # il doit contenir les lignes
    # From: expéditeur
    # To: destinataire
    # ligne vide
    # Message
    # .
    data="From: %s\r\nTo: %s\r\n%s\r\n.\r\n" % (expediteur,destinataire,message)
    # envoi message
    erreur=sendCommand(connexion,data,verbose,0)
    if erreur :
        connexion.close()
        return erreur
    # cmde quit
    erreur=sendCommand(connexion,"QUIT",verbose,1)
    if erreur :
        connexion.close()
        return erreur
    # fin
    connexion.close()
    return "Message envoye"

# --------------------------------------------------------------------------
def sendCommand(connexion,commande,verbose,withRCLF):
    # envoie commande dans le canal connexion
    # mode verbeux si verbose=1
    # si withRCLF=1, ajoute la séquence RCLF à commande

    # donn?es
    RCLF="\r\n" if withRCLF else ""
    # envoi cmde si commande non vide
    if commande:
        connexion.send("%s%s" % (commande,RCLF))
        # écho éventuel
        if verbose:
            affiche(commande,1)
        # lecture réponse de moins de 1000 caractères
        reponse=connexion.recv(1000)
        # écho éventuel
        if verbose:
            affiche(reponse,2)
        # récupération code erreur
        codeErreur=reponse[0:3]
        # erreur renvoyée par le serveur ?
        if int(codeErreur) >=500:
            return reponse[4:]
        # retour sans erreur
        return ""

# --------------------------------------------------------------------------
def affiche(echange,sens):
    # affiche échange ? l'écran
    # si sens=1 affiche -->echange
    # si sens=2 affiche <-- échange sans les 2 derniers caractères RCLF
    if sens==1:
        print "--> [%s]" % (echange)
        return
    elif sens==2:
        l=len(echange)
        print "<-- [%s]" % echange[0:l-2]
        return

      
# --------------------------------------------------------------------------
def cutNewLineChar(ligne):
    # on supprime la marque de fin de ligne de [ligne] si elle existe
    l=len(ligne)
    while(ligne[l-1]=="\n" or ligne[l-1]=="\r"):
        l-=1
    return(ligne[0:l])

# main ----------------------------------------------------------------
  
 # client SMTP (SendMail Transfer Protocol) permettant d'envoyer un message
# les infos sont prises dans un fichier INFOS contenant les lignes suivantes
# ligne 1 : smtp, expéditeur, destinataire
# lignes suivantes : le texte du message

# expéditeur:email expéditeur
# destinataire: email destinataire
# smtp: nom du serveur smtp à utiliser


# protocole de communication SMTP client-serveur
# -> client se connecte sur le port 25 du serveur smtp
# <- serveur lui envoie un message de bienvenue
# -> client envoie la commande EHLO: nom de sa machine
# <- serveur répond OK ou non
# -> client envoie la commande mail from: <exp?diteur>
# <- serveur répond OK ou non
# -> client envoie la commande rcpt to: <destinataire>
# <- serveur répond OK ou non
# -> client envoie la commande data
# <- serveur répond OK ou non
# -> client envoie ttes les lignes de son message et termine avec une ligne contenant le seul caract?re .
# <- serveur répond OK ou non
# -> client envoie la commande quit
# <- serveur répond OK ou non

# les réponses du serveur ont la forme xxx texte où xxx est un nombre à 3 chiffres. Tout nombre xxx >=500
# signale une erreur. La réponse peut comporter plusieurs lignes commençant toutes par xxx- sauf la dernière
# de la forme xxx(espace)

# les lignes de texte échangées doivent se terminer par les caractéres RC(#13) et LF(#10)

# # les paramètres de l'envoi du courrier
MAIL="mail2.txt" 
# on récupère les paramètres du courrier
res=getInfos(MAIL)
# erreur ?
if res[0]:
      print "%s" % (erreur)
      sys.exit()
# envoi du courrier en mode verbeux
(smtpServer,expediteur,destinataire,message)=res[1:]
print "Envoi du message [%s,%s,%s]" % (smtpServer,expediteur,destinataire)
resultat=sendmail(smtpServer,expediteur,destinataire,message,True)
print "Resultat de l'envoi : %s" % (resultat)
# fin
sys.exit()

